import gleam/dynamic
import gleam/int
import gleam/json
import gleam/list
import gleam/option
import gleam/result
import lustre
import lustre/attribute
import lustre/effect.{type Effect}
import lustre/element.{type Element}
import lustre/element/html
import lustre/event
import rsvp
import user_state

pub fn main() {
  // For hydration, we'll read the initial model from the window
  let flags = get_initial_state()

  let app = lustre.application(init, update, view)
  let assert Ok(_) = lustre.start(app, "#app", flags)

  Nil
}

fn get_initial_state() -> user_state.UserState {
  user_state.UserState(0, option.Some(1))
}

type Screen {
  Title
  HostControl
  Display
  Player
}

type ApplicationState {
  ApplicationState(user: user_state.UserState, new_game: String, screen: Screen)
}

fn init(user: user_state.UserState) -> #(ApplicationState, Effect(Msg)) {
  let application_state = ApplicationState(user:, new_game: "", screen: Title)
  #(application_state, effect.none())
}

type Msg {
  HostGame
  JoinGame
  UserTypedGameID(String)
  Quit
}

fn update(state: ApplicationState, msg: Msg) -> #(ApplicationState, Effect(Msg)) {
  case msg {
    JoinGame -> #(
      ApplicationState(
        user: user_state.UserState(..state.user, game_id: option.Some(0)),
        new_game: "",
        screen: Player,
      ),
      effect.none(),
    )
    HostGame -> #(ApplicationState(..state, screen: Display), effect.none())
    UserTypedGameID(id) -> #(
      ApplicationState(..state, new_game: id),
      effect.none(),
    )
    Quit -> #(ApplicationState(..state, screen: Title), effect.none())
  }
}

fn view(state: ApplicationState) -> Element(Msg) {
  let styles = [
    #("max-width", "30ch"),
    #("margin", "0 auto"),
    #("display", "flex"),
    #("flex-direction", "column"),
    #("gap", "1em"),
  ]

  case state.screen {
    Title ->
      html.div([attribute.styles(styles)], [
        html.h1([], [html.text("Title")]),
        view_title(state.new_game),
      ])
    Player ->
      html.div([attribute.styles(styles)], [
        html.h1([], [html.text("Game")]),
        view_quit_button(),
      ])
    _ ->
      html.div([attribute.styles(styles)], [
        html.h1([], [html.text("Lol LMAO")]),
      ])
  }
}

fn view_title(game_name: String) -> Element(Msg) {
  html.div([], [
    html.input([
      attribute.placeholder("Enter game id to join:"),
      attribute.value(game_name),
      event.on_input(UserTypedGameID),
    ]),
    html.button([event.on_click(JoinGame)], [html.text("Join")]),
  ])
}

fn view_quit_button() -> Element(Msg) {
  html.div([], [html.button([event.on_click(Quit)], [html.text("Quit")])])
}
